%The code produces a single realization of the following RNIC model for the
%Girsanov method.
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 
%
%              with KI = 1, KE = 0.01.  So that KF = 0 and KC = 0.
%
%                    Chemistry:  0 -[kb]-> S,
%                                S -[kd]-> 0,
%                                
%
%          with rates kb = 1 and kd = 0.1.  
%          These are the rates from the paper by Duso and Zechner. We take
%          the derivative with respect to kb.
%
%           Another important parameter is the mean of the input
%           distribution, which is Poisson(lambda).  We take:
%
%           lambda = 10;
%
%The outpus are the following:
%
%                     x = output vector
%
%                     nrv = number of random variables utilized.
%
%                     W = accumulated weight function for a derivative
%
%Written by David F. Anderson, June 4th, 2024.

function [x,nrv,W]= StandardGillespieGirsanov(tend,xin,parameters)

%weight function
W = 0;
%max number of steps
N=10^3;

kb = parameters(5);
kd = parameters(6);

x = xin;
times=zeros([1,N+1]);
nrv=0;

for i = 1:N
    
    a1 = kb;
    a2 = kd*x(1);
    
    a0 = a1+a2;
    
    r = rand;
    nrv=nrv+1;
    
    Delta = log(1/r)/a0;
    
    times(1,i+1)=times(1,i)+Delta;
    if times(1,i+1)>tend
        W = W - (tend - times(1,i))*a1/kb;
        break
    else

        r2=rand;
        nrv=nrv+1;

        if r2<a1/a0
            x(1) = x(1) + 1;
            W = W + 1/kb;

        else
            x(1) = x(1) - 1;
        end
        W = W - Delta*a1/kb;
        
    end
    
end

if i >= N
    'Too big in Gillespie'
end