%The code produces a single realization of the following chemical model.
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 
%
%              with KI = 1, KE = 0.01.  So that KF = 0 and KC = 0.
%
%                    Chemistry:  0 -[kb]-> S,
%                                S -[kd]-> 0,
%                                
%
%          with rates kb = 1 and kd = 0.1.  
%          These are the rates from the paper by Duso and Zechner.  
%
% Usage [x,nrv]= StandardGillespie(tend,xin,parameters)
%
%               parameters = the parameters of the model.
%                            It is a vector, but only use  the 5th and 6th
%                            elements (the others are for the compartment)
%               tend = end time. Real.
%               xin = initial condition.  1 x 1vector.  S molecule
%
%
%The outpus are the following:
%
%                     x = output vector (final state)
%
%                     nrv = number of random variables utilized.
%
%
%Written by David F. Anderson, June 25th, 2024.

function [x,nrv]= StandardGillespie(tend,xin,parameters)


%max number of steps
N=10^3;

kb = parameters(5);
kd = parameters(6);

x = xin;
times=zeros([1,N+1]);
nrv=0;

for i = 1:N
    
    a1 = kb;
    a2 = kd*x(1);
    
    a0 = a1+a2;
    
    r = rand;
    nrv=nrv+1;
    
    Delta = log(1/r)/a0;
    
    times(1,i+1)=times(1,i)+Delta;
    if times(1,i+1)>tend
        break
    else

        r2=rand;
        nrv=nrv+1;

        if r2<a1/a0
            x(1) = x(1) + 1;

        else
            x(1) = x(1) - 1;
        end
        
    end
    
end

if i >= N
    'Too big in Gillespie'
end